@@ -81,7 +81,7 @@ use constant {    ## no critic (ValuesAndExpressions::ProhibitConstantPragma, Na
   LOG_FORMAT                => " %s [%s] %s\n",
 };
 
-our $VERSION = '1.1.1';
+our $VERSION = '1.1.2';
 
 use vars qw{ @EXPORT_OK };
 
@@ -982,6 +982,7 @@ sub _crypt {
   if ( $self->get_encryption ) {
     $cipher = Crypt::CBC->new(
       '-pass'        => $passkey,
+      '-key'         => $passkey,
       '-cipher'      => $self->get_cipher,
       '-nodeprecate' => 1,
     );
@@ -1059,11 +1060,12 @@ __END__
 
 =head1 NAME
 
- Amazon::Credentials
+Amazon::Credentials - fetch Amazon credentials from file, environment or role
 
 =head1 SYNOPSIS
 
- my $aws_creds = Amazon::Credentials->new({order => [qw/env file container role/]});
+ my @order = qw{ env file container role };
+ my $creds = Amazon::Credentials->new( { order => \@order } );
 
 =head1 DESCRIPTION
 
@@ -1075,6 +1077,11 @@ to determine the order in which the class will look for credentials.
 The default order is I<environent>, I<file>, I<container>, I<instance
 meta-data>. See L</new>.
 
+=head1 VERSION
+
+This document reverse to verion @PACKAGE_VERION@ of
+C<Amazon::Credentials>.
+
 =head1 METHODS AND SUBROUTINES
 
 =head2 new
@@ -1164,7 +1171,7 @@ your credentials.>
 
 =item file - Configuration Files
 
-=over 15
+=over 10
 
 =item ~/.aws/config
 
@@ -1214,8 +1221,6 @@ Example:
 
   my $creds = Amazon::Credentials->new( { order => [ qw/file env role/] });
 
-=over 10
-
 =item passkey
 
 A custom passkey for encryption. You can pass a scalar or a reference
@@ -1262,8 +1267,6 @@ expires.
    $aws_creds->refresh_token()
  }
 
-=back
-
 =item timeout
 
 When looking for credentials in metadata URLs, this parameter
@@ -1324,7 +1327,51 @@ the same manner as the constructor.
 =head2 get_creds_from_*
 
 These methods are called internally when the C<new> constructor is
-invoked. You should not normally call these methods
+invoked. You should never need to call these methods. All of these
+methods will return a hash of credential information and metadata
+described below.
+
+=over 5
+
+=item aws_access_key_id
+
+The AWS access key.
+
+=item aws_secret_access_key
+
+The AWS secret key.
+
+=item token
+
+Security token used with access keys.
+
+=item expiration
+
+Token expiration date.
+
+=item role
+
+IAM role if available.
+
+=item source
+
+The source from which the credentials were found. 
+
+=over 3
+
+=item * IAM - retrieved from container or instance role
+
+=item * container - 'ECS' if retrieved from container
+
+=item * file - retrieved from file
+
+=item * process - retrieved from an external process
+
+=item * ENV - retrieved from environment
+
+=back
+
+=back
 
 =head3 get_creds_from_container
 
@@ -1354,41 +1401,13 @@ file. Returns the credentials tuple.
 
 Returns a hash, possibly containing access keys and a token.
 
-=over 5
-
-=item aws_access_key_id
-
-The AWS access key.
-
-=item aws_secret_access_key
-
-The AWS secret key.
-
-=item token
-
-Security token used with access keys.
-
-=item expiration
-
-Token expiration date.
-
-=item role
-
-IAM role if available.
-
-=item source
-
-Will be 'IAM' if role and credentials found.
-
-=back
-
 =head2 get_default_region
 
-Returns the region of the currently running instance.  The constructor
-will set the region to this value unless you set your own C<region>
-value.  Use C<get_region> to retrieve the value after instantiation or
-you can call this method again and it will make a second call to
-retrieve the instance metadata.
+Returns the region of the currently running instance or container.
+The constructor will set the region to this value unless you set your
+own C<region> value. Use C<get_region> to retrieve the value after
+instantiation or you can call this method again and it will make a
+second call to retrieve the instance metadata.
 
 =head2 get_ec2_credentials (deprecated)
 
@@ -1406,15 +1425,16 @@ available I<at least> 5 minutes before a token expires.
 
 =head2 reset_credentials
 
-By default this method will remove credentials from cache if you pass
-a false or no value. Passing a true value will refresh your
+By default this method will remove credentials from the cache if you
+pass a false or no value. Passing a true value will refresh your
 credentials from the original source (equivalent to calling
-C<find_credentials>).
+C<set_credentials>).
 
-=head2 refresh_token
+=head2 refresh_token (deprecated)
 
- refresh_token() (deprecated)
- refresh_credentials()
+use C<refresh_credentials()>
+
+=head2 refresh_credentials()
 
 Retrieves a fresh set of IAM credentials.
 
@@ -1422,21 +1442,24 @@ Retrieves a fresh set of IAM credentials.
    $creds->refresh_token()
  }
 
+=head2 set_credentials
+
+Looks for your credentials according to the order specified by the
+C<order> attribute passed in the constructor and stores the
+credentials in the cache.
+
+I<Note that you should never have to call
+this method. If you call this method it will ignore your cache
+setting!>
+
 =head1 SETTERS/GETTERS
 
-All of the parameters described in the new method can be accessed by a
-I<getter> or set using a I<setter>.
+All of the options described in the new method can be accessed by a
+I<getter> or set using a I<setter> of the same name.
 
-=head1 CONTRIBUTING
+Example:
 
-You can find this project on GitHub at
-L<https://github.com/rlauer6/perl-Amazon-Credentials>.  PRs are always
-welcomed!
-
-=head1 VERSION
-
-This document reverse to verion @PACKAGE_VERION@ of
-C<Amazon::Credentials>.
+ $creds->set_cache(0);
 
 =head1 DIAGNOSTICS
 
@@ -1449,7 +1472,7 @@ careful not to expose these values in logs.>
 =head1 CONFIGURATION AND ENVIRONMENT
 
 The module will recognize several AWS specific environment variables
-describe throughout this documentation.
+described throughout this documentation.
 
 =over 5
 
@@ -1480,29 +1503,27 @@ define a role in your credentials file thusly:
 
 The module will not return credentials for the I<developer>
 profile. While it would be theoretically possible to return those
-credentials, in order to assume a role, one needs credentials.
+credentials, in order to assume a role, one needs credentials (chicken
+and egg problem).
 
 =head1 DEPENDENCIES
 
-Dependencies as report by C<scandeps.pl>...however lower versions of
-these modules are probably acceptable.
+Lower versions of these modules may be acceptable.
 
- 'Carp'                          => '1.26',
- 'Class::Accessor::Fast'         => '0.31',
- 'Config::Tiny'                  => '2.28',
- 'Data::Dumper'                  => '2.145',
- 'Date::Format'                  => '2.24',
- 'Exporter'                      => '5.68',
- 'File::HomeDir'                 => '1.00',
- 'File::chdir'                   => '0.1010',
- 'HTTP::Request'                 => '6.00',
- 'JSON::PP'                      => '2.97001',
- 'LWP::UserAgent'                => '6.36',
- 'POSIX::strptime'               => '0.13',
- 'Scalar::Util'                  => '1.50',
- 'Time::Local'                   => '1.2300',
- 'constant'                      => '1.33',
- 'parent'                        => '0.225',
+ 'Class::Accessor::Fast' => '0.31'
+ 'Config::Tiny'          => '2.28'
+ 'Date::Format'          => '2.24'
+ 'File::HomeDir'         => '1.00'
+ 'File::chdir'           => '0.1010'
+ 'HTTP::Request'         => '6.00'
+ 'List::Util'            => '1.5'
+ 'LWP::UserAgent'        => '6.36'
+ 'POSIX::strptime'       => '0.13'
+
+...and possibly others
+
+In order to enable true encryption of your credentials when cached,
+C<Crypt::CBC> is also required.
 
 =head1 SECURITY CONSIDERATIONS
 
@@ -1558,7 +1579,7 @@ There are two ways your programs typically will fetch the keys; either
 using the getter methods on the individual credentials keys or by
 retrieving a hash containing all of the keys.
 
-=over 10
+=over 5
 
 =item C<credential_keys()>
 
@@ -1596,7 +1617,7 @@ passkey and method used are known to a determined bad actor is
 no better than obfuscation. Accordingly, there are several ways you
 can and should encrypt credentials in a more secure way.
 
-=over 10
+=over 3
 
 =item Using a Custom C<passkey>
 
@@ -1616,19 +1637,22 @@ credentials or your methods for encryption in logs...better but
 not perfect. It's still possible to expose your passkey and your
 credentials if you are not careful.
 
- my $passkey = Amazon::Credentials::create_passkey;
+ use Amazon::Credentials qw{ create_passkey };
+
+ my $passkey = create_passkey();
 
  my $credentials = Amazon::Credentials( passkey => sub { return $passkey } );
 
 A more secure approach would be for your subroutine to retrieve a
-passkey from a source other than your own program and B<never>
-store the passkey inside your program.
+passkey from a source other than your own program and B<never> store
+the passkey inside your program.
 
 =item Using a Custom Cipher
 
-As noted, the default cipher is used for encrypting your credentials,
-however you can pass a custom cipher further obfuscating the methods
-used to encrypt your credentials.
+As noted, the default C<Crypt::CBC> cipher is used for encrypting your
+credentials, however you can pass a custom cipher supported by
+C<Crypt::CBC> further obfuscating the methods used to encrypt your
+credentials.
 
  my $credentials = Amazon::Credentials(
    passkey => \&fetch_passkey,
@@ -1662,6 +1686,37 @@ continue to provide the old passkey before you can reset the passkey.
  
  $passkey = $credentials->rotate_credentials(get_passkey(1));
 
+=item Using Custom Encryption Methods
+
+Finally, you can also provide your own C<encrypt()> and C<decrypt()>
+methods when you call the C<new()> constructor. These methods will be
+passed the string to encrypt or decrypt and the passkey. Your methods
+should return the decrypted or encrypted strings. Your methods can
+ignore the passkey if your methods provide their own passkey or
+mechanisms for encryption.
+
+ use Amazon::Credentials qw( create_passkey };
+
+ my $passkey = create_passkey();
+
+ sub my_encrypt {
+   my ($self, $str) = @_;
+
+   ...
+   return $encrypted_str;
+ }
+
+ sub my_decrypt {
+
+   ...
+   return $deecrypted_str;
+ }
+
+ my $creds = Amazon::Credentials->new( encrypt => \&my_encrypt,
+                                       decrypt => \&my_decrypt,
+                                       passkey => sub { return $passkey },
+                                     );
+
 =back
 
 =back
@@ -1728,6 +1783,12 @@ that you might pass to C<Amazon::Credentials>.
 
 This module has not been tested on Windows OS.
 
+=head1 CONTRIBUTING
+
+You can find this project on GitHub at
+L<https://github.com/rlauer6/perl-Amazon-Credentials>.  PRs are always
+welcomed!
+
 =head1 LICENSE AND COPYRIGHT
 
 This module is free software. It may be used, redistributed and/or
